@@ -4,6 +4,7 @@
 
 import _thread
 import argparse
+import json
 import os
 import re
 import sys
@@ -19,20 +20,96 @@ from Annotations2Sub import version
 from Annotations2Sub.Annotation import Parse
 from Annotations2Sub.Convert import Convert
 from Annotations2Sub.Sub import Sub
-from Annotations2Sub.tools import (
-    AnnotationsForArchive,
-    CheckUrl,
+from Annotations2Sub.utils import (
+    Flags,
+    MakeSureStr,
     RedText,
     Stderr,
-    VideoForInvidious,
     YellowText,
     _,
-    Flags,
+    urllibWapper,
 )
 
 
-def run():
-    """程序入口"""
+def Dummy(*args, **kwargs):
+    pass
+
+
+def run(argv=None):
+    """跑起来🐎🐎🐎"""
+
+    def CheckUrl(url: str = "https://google.com/", timeout: float = 3.0) -> bool:
+        """检查网络"""
+        try:
+            urllib.request.urlopen(url=url, timeout=timeout)
+        except:
+            return False
+        return True
+
+    def MediaFromInvidious(videoId: str, instanceDomain: str = "") -> tuple:
+        """返回一个视频流和音频流网址"""
+        instances = []
+        if instanceDomain != "":
+            instances.append([instanceDomain])
+        if instanceDomain == "":
+            instances = json.loads(
+                urllibWapper("https://api.invidious.io/instances.json")
+            )
+        for instance in instances:
+            try:
+                if not instance[1]["api"]:  # type: ignore
+                    continue
+            except:
+                pass
+            domain = instance[0]
+            url = f"https://{domain}/api/v1/videos/{videoId}"
+            Stderr(_("获取 {}").format(url))
+            try:
+                data = json.loads(urllibWapper(url))
+            except:
+                continue
+            videos = []
+            audios = []
+            for i in data.get("adaptiveFormats"):
+                if re.match("video", i.get("type")) is not None:
+                    videos.append(i)
+                if re.match("audio", i.get("type")) is not None:
+                    audios.append(i)
+            videos.sort(key=lambda x: int(x.get("bitrate")), reverse=True)
+            audios.sort(key=lambda x: int(x.get("bitrate")), reverse=True)
+            video = MakeSureStr(videos[0]["url"])
+            audio = MakeSureStr(audios[0]["url"])
+            return video, audio
+        raise Exception
+
+    def AnnotationsFromArchive(videoId: str) -> str:
+        # 移植自 https://github.com/omarroth/invidious/blob/ea0d52c0b85c0207c1766e1dc5d1bd0778485cad/src/invidious.cr#L2835
+        # 向 https://archive.org/details/youtubeannotations 致敬
+        # 如果你对你的数据在意, 就不要把它们托付给他人
+        # Rain Shimotsuki 不仅是个打歌词的, 他更是一位创作者
+        # 自己作品消失, 我相信没人愿意看到
+        """返回注释在互联网档案馆的网址"""
+        ARCHIVE_URL = "https://archive.org"
+        CHARS_SAFE = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"
+
+        if re.match(r"[a-zA-Z0-9_-]{11}", videoId) is None:
+            raise ValueError("Invalid videoId")
+
+        index = CHARS_SAFE.index(videoId[0]).__str__().rjust(2, "0")
+
+        # IA doesn't handle leading hyphens,
+        # so we use https://archive.org/details/youtubeannotations_64
+        if index == "62":
+            index = "64"
+            videoId.replace("^-", "A")
+
+        file = f"{videoId[0:3]}/{videoId}.xml"
+
+        return f"{ARCHIVE_URL}/download/youtubeannotations_{index}/{videoId[0:2]}.tar/{file}"
+
+    Dummy([CheckUrl, AnnotationsFromArchive, MediaFromInvidious])  # type: ignore
+
+    code = 0
     parser = argparse.ArgumentParser(description=_("下载和转换 Youtube 注释"))
     parser.add_argument(
         "queue",
@@ -147,7 +224,7 @@ def run():
         help=_("显示更多消息"),
     )
 
-    args = parser.parse_args()
+    args = parser.parse_args(argv)
 
     if args.verbose:
         Flags.verbose = True
@@ -155,18 +232,10 @@ def run():
     if args.output_to_stdout:
         if args.output_directory is not None:
             Stderr(RedText(_("--output-to-stdout 不能与 --output-directory 选项同时使用")))
-            exit(1)
-        if args.no_overwrite_files:
-            Stderr(RedText(_("--output-to-stdout 不能与 --no-overwrite-files 选项同时使用")))
-            exit(1)
+            return 1
         if args.output is not None:
             Stderr(RedText(_("--output-to-stdout 不能与 --output 选项同时使用")))
-            exit(1)
-        if args.download_annotation_only:
-            Stderr(
-                RedText(_("--output-to-stdout 不能与 --download-annotation-only 选项同时使用"))
-            )
-            exit(1)
+            return 1
         if args.preview_video or args.generate_video:
             Stderr(
                 RedText(
@@ -175,44 +244,26 @@ def run():
                     )
                 )
             )
-            exit(1)
-
-    if args.no_keep_intermediate_files:
-        if args.download_annotation_only:
-            Stderr(
-                RedText(
-                    _(
-                        "--no-keep-intermediate-files 不能与 --download-annotation-only 选项同时使用"
-                    )
-                )
-            )
-            exit(1)
-        if not (args.download_for_archive or args.preview_video or args.generate_video):
-            Stderr(
-                RedText(
-                    _(
-                        "--no-keep-intermediate-files 必须和 --download-for-archive 或 --preview-video 或 --generate-video 选项使用"
-                    )
-                )
-            )
-            exit(1)
+            return 1
 
     if args.output is not None:
         if args.output_directory is not None:
             Stderr(RedText(_("--output 不能与 --output--directory 选项同时使用")))
-            exit(1)
+            return 1
         if len(args.queue) > 1:
             Stderr(RedText(_("--output 只能处理一个文件")))
-            exit(1)
+            return 1
 
     if args.output_directory is not None:
         if os.path.isdir(args.output_directory) is False:
             Stderr(RedText(_("转换后文件输出目录应该指定一个文件夹")))
-            exit(1)
+            return 1
 
     if args.preview_video or args.generate_video:
         args.download_for_archive = True
         args.embrace_libass = True
+        if args.invidious_instances == None:
+            args.invidious_instances = ""
 
     if args.embrace_libass and (
         args.transform_resolution_x == 100 or args.transform_resolution_y == 100
@@ -237,11 +288,14 @@ def run():
         _thread.start_new_thread(CheckNetwork, ())
 
     for Task in args.queue:
-        videoId = Task
+        videoId = MakeSureStr(Task)
         annotationFile = Task
         if args.download_for_archive:
+            if videoId.startswith("\\"):
+                videoId = videoId.replace("\\", "", 1)
             if re.match(r"[a-zA-Z0-9_-]{11}", videoId) is None:
                 Stderr(RedText(_("{} 不是一个有效的视频 ID").format(videoId)))
+                code = 1
                 continue
 
             annotationFile = f"{videoId}.xml"
@@ -256,10 +310,12 @@ def run():
                     Stderr(YellowText(_("文件已存在, 跳过下载 ({})").format(videoId)))
                     skipDownload = True
             if not skipDownload:
-                url = AnnotationsForArchive(videoId)
+                url = AnnotationsFromArchive(videoId)
                 Stderr(_("下载 {}").format(url))
-                with urllib.request.urlopen(url) as r:
-                    string = r.read().decode("utf-8")
+                string = urllibWapper(url)
+                if args.output_to_stdout:
+                    print(string, file=sys.stdout)
+                    continue
                 with open(annotationFile, "w", encoding="utf-8") as f:
                     f.write(string)
 
@@ -268,13 +324,15 @@ def run():
 
         if os.path.isfile(annotationFile) is False:
             Stderr(RedText(_("{} 不是一个文件").format(annotationFile)))
+            code = 1
             continue
 
         with open(annotationFile, "r", encoding="utf-8") as f:
             annotationsString = f.read()
 
         if annotationsString == "":
-            Stderr(YellowText(_("{} 可能没有 Annotation").format(Task)))
+            Stderr(YellowText(_("{} 可能没有 Annotation").format(videoId)))
+            code = 1
             continue
 
         try:
@@ -283,10 +341,12 @@ def run():
             Stderr(RedText(_("{} 不是一个有效的 XML 文件").format(annotationFile)))
             if Flags.verbose:
                 Stderr(traceback.format_exc())
+            code = 1
             continue
 
         if tree.find("annotations") == None:
             Stderr(RedText(_("{} 不是 Annotation 文件").format(annotationFile)))
+            code = 1
             continue
 
         if len(tree.find("annotations").findall("annotation")) == 0:
@@ -320,28 +380,26 @@ def run():
         sub.events.extend(events)
         sub.info["PlayResX"] = args.transform_resolution_x
         sub.info["PlayResY"] = args.transform_resolution_y
-        sub.info["Title"] = annotationFile
+        sub.info["Title"] = os.path.basename(annotationFile)
         sub.styles["Default"].Fontname = args.font
         subString = sub.Dump()
         if args.output_to_stdout:
-            subFile = ""
             print(subString, file=sys.stdout)
+            continue
+        noSave = False
         if args.no_overwrite_files:
             if os.path.exists(subFile):
                 Stderr(YellowText(_("文件已存在, 跳过输出 ({})").format(subFile)))
-                subFile = ""
+                noSave = True
         if args.no_keep_intermediate_files:
             os.remove(annotationFile)
             Stderr(_("删除 {}").format(annotationFile))
-        if subFile != "":
+        if not noSave:
             with open(subFile, "w", encoding="utf-8") as f:
                 f.write(subString)
             Stderr(_("保存于: {}").format(subFile))
 
-        if args.preview_video:
-            # 从 Invidious 获取视频流和音频流, 并塞给 mpv, FFmpeg
-            video, audio = VideoForInvidious(videoId, args.invidious_instances)
-            cmd = rf'mpv "{video}" --audio-file="{audio}" --sub-file="{subFile}"'
+        def d1():
             if Flags.verbose:
                 Stderr(cmd)
             exit_code = os.system(cmd)
@@ -352,15 +410,16 @@ def run():
                 os.remove(subFile)
                 Stderr(_("删除 {}").format(subFile))
 
+        video = audio = ""
+        if args.preview_video or args.generate_video:
+            video, audio = MediaFromInvidious(videoId, args.invidious_instances)
+
+        if args.preview_video:
+            cmd = rf'mpv "{video}" --audio-file="{audio}" --sub-file="{subFile}"'
+            d1()
+
         if args.generate_video:
-            video, audio = VideoForInvidious(videoId, args.invidious_instances)
             cmd = rf'ffmpeg -i "{video}" -i "{audio}" -vf "ass={subFile}" {subFile}.mp4'
-            if Flags.verbose:
-                Stderr(cmd)
-            exit_code = os.system(cmd)
-            if Flags.verbose:
-                if exit_code != 0:
-                    Stderr(YellowText("exit with {}".format(exit_code)))
-            if args.no_keep_intermediate_files:
-                os.remove(subFile)
-                Stderr(_("删除 {}").format(subFile))
+            d1()
+
+    return code
