@@ -2,19 +2,16 @@
 # -*- coding: utf-8 -*-
 
 import os
-
-
 import xml.etree.ElementTree
 
 from Annotations2Sub.Annotation import Parse
 from Annotations2Sub.Convert import Convert
-from Annotations2Sub.tools import Flags
 from Annotations2Sub.Sub import Sub
+from Annotations2Sub.utils import Flags
 
 
 def test_Annotations2Sub():
     Flags.verbose = True
-    Flags.unstable = True
     filePath = os.path.join(os.path.dirname(__file__), "xml.test")
     with open(filePath, "r", encoding="utf-8") as f:
         string = f.read()
